import java.util.Date;
import java.text.SimpleDateFormat;
import com.ub.misc.epak.EPakSigner;

/**
 * eLink request signing, signature validation and key pair generation exapmle.
 *
 * Requirements to run this example:
 * 1) eLink data signer component - com.ub.misc.epak.EPakSigner;
 * 2) Bouncy Castle Crypto APIs librarry.
 *    It's available at http://www.bouncycastle.org/latest_releases.html
 *
 * @author SEB banka, Aldis Egle, 2008
 */
public class SignExample {

  public static void main (String[] args) {
    try {
      // Generating new public and secure key pair
      EPakSigner si = new EPakSigner();
      String certPem = si.generatePair("SEBUB", "IT", "SEB banka", "LV", 100);
      String privKey = si.getPrivateKey();
      System.out.println("Certificate:\n" + certPem);
      System.out.println("Secure key:\n" + privKey);
      // Data for request "0001" (user authentication from Ibanka)
      Date date = new Date();
      String[] buf = new String[8];
      buf[0] = "SEBUB";        // IB_SND_ID
      buf[1] = "0001";         // IB_SERVICE
      buf[2] = "LTKNET";       // IB_REC_ID
      buf[3] = "311276-11284"; // IB_USER
      buf[4] = (new SimpleDateFormat("dd.MM.yyyy")).format(date); // IB_DATE
      buf[5] = (new SimpleDateFormat("HH:mm:ss")).format(date);   // IB_TIME
      buf[6] = "ID=311276-11284;NAME=JNIS BRZI";              // IB_USER_INFO
      buf[7] = "001";          // IB_VERSION
      // Dispalying data
      System.out.println("Request data:");
      for (int i = 0; i < buf.length; i++)
        System.out.println("  buf["+i+"]="+buf[i]);
      // Creating signature
      String signature = signWithKey(privKey, buf);
      System.out.println("\nSignature: " + signature + "\n");
      // Verifying signature
      boolean isValid = isSignatureValid(certPem, buf, signature);
      System.out.println("Signature is valid: " + isValid);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  /** Creating signature */
  private static String signWithKey(String privKey, String[] buf) throws Exception {
    EPakSigner si = new EPakSigner();
    si.beginSign(privKey);
    for (int i = 0; i < buf.length; i++)
      si.add(buf[i]);
    return si.endSign();
  }

  /** Verifying signature */
  private static boolean isSignatureValid(String certPem, String[] buf, String signature)
    throws Exception
  {
    EPakSigner si = new EPakSigner();
    si.beginVerify(certPem);
    for (int i = 0; i < buf.length; i++)
      si.add(buf[i]);
    return si.endVerify(signature);
  }

}